from collections.abc import Callable
from os.path import dirname
from pathlib import Path

import pytest

ResourceFunc = Callable[[str], Path]


@pytest.fixture()
def resource() -> ResourceFunc:
    def _resource(name: str) -> Path:
        data_dir = Path(dirname(__file__)) / "data"
        return data_dir / name

    return _resource



class Hash:
    def bits(self) -> list[list[bool]]:
        """
        Returns a 2D lists of bools that represent the bit-matrix
        behind the hash.

        Returns:
            list[list[bool]]: The 2D matrix of bits
        """
        ...

    def hex(self) -> str:
        """
        Returns the hexadecimal encoded representation of the hash.

        Returns:
            str: The hexadeicmal encoded hash
        """
        ...

def average_hash(path: str, width: int = 8, height: int = 8) -> Hash:
    """
    Generates the average hash for an image at the provided path.

    Args:
        path (str): The path of the image
        width (int, optional): The width of the resulting bit matrix. Defaults to 8.
        height (int, optional): The height of the resulting bit matrix. Defaults to 8.

    Returns:
        Hash: An object representing the hash
    """
    ...

def difference_hash(path: str, width: int = 8, height: int = 8) -> Hash:
    """
    Generates the difference hash for an image at the provided path.

    Args:
        path (str): The path of the image
        width (int, optional): The width of the resulting bit matrix. Defaults to 8.
        height (int, optional): The height of the resulting bit matrix. Defaults to 8.

    Returns:
        Hash: An object representing the hash
    """
    ...

def perceptual_hash(path: str, width: int = 8, height: int = 8) -> Hash:
    """
    Generates the perceptual hash for an image at the provided path.

    Args:
        path (str): The path of the image
        width (int, optional): The width of the resulting bit matrix. Defaults to 8.
        height (int, optional): The height of the resulting bit matrix. Defaults to 8.

    Returns:
        Hash: An object representing the hash
    """
    ...



import pytest
from conftest import ResourceFunc
from imghash import perceptual_hash


def test_perceptual_hash(resource: ResourceFunc):
    # Arrange
    img_path = resource("test.png")

    # Act
    hash = perceptual_hash(img_path.absolute().as_posix())

    # Assert
    assert hash
    assert hash.hex() == "acdbe86135344e3a"
    assert hash.bits()


def test_perceptual_hash_with_txt_file(resource: ResourceFunc):
    # Arrange
    img_path = resource("test.txt")

    # Act
    with pytest.raises(ValueError) as e:
        perceptual_hash(img_path.absolute().as_posix())

    # Assert
    assert e.match("image format")



import pytest
from conftest import ResourceFunc
from imghash import average_hash


def test_average_hash(resource: ResourceFunc):
    # Arrange
    img_path = resource("test.png")

    # Act
    hash = average_hash(img_path.absolute().as_posix())

    # Assert
    assert hash
    assert hash.hex() == "ffffff0e00000301"
    assert hash.bits()


def test_average_hash_with_txt_file(resource: ResourceFunc):
    # Arrange
    img_path = resource("test.txt")

    # Act
    with pytest.raises(ValueError) as e:
        average_hash(img_path.absolute().as_posix())

    # Assert
    assert e.match("image format")
